@@ -450,27 +450,30 @@ const rScriptLineSplit = /\r?\n/;
 const rScriptContinuation = /\\\s*$/;
 const rScriptComment = /^\s*(?:#.*)?$/;
 const rScriptAssignment = /^\s*(?<name>[A-Za-z_]\w*)\s*=\s*(?<expr>.+)$/;
+const rPartComment = '\\s*(#.*)?$';
 const rScriptFunctionBegin = new RegExp(
     '^(?<async>\\s*async)?\\s*function\\s+(?<name>[A-Za-z_]\\w*)\\s*\\(' +
-        '\\s*(?<args>[A-Za-z_]\\w*(?:\\s*,\\s*[A-Za-z_]\\w*)*)?(?<lastArgArray>\\s*\\.\\.\\.)?\\s*\\)\\s*:\\s*$'
+        `\\s*(?<args>[A-Za-z_]\\w*(?:\\s*,\\s*[A-Za-z_]\\w*)*)?(?<lastArgArray>\\s*\\.\\.\\.)?\\s*\\)\\s*:${rPartComment}`
 );
 const rScriptFunctionArgSplit = /\s*,\s*/;
-const rScriptFunctionEnd = /^\s*endfunction\s*$/;
-const rScriptLabel = /^\s*(?<name>[A-Za-z_]\w*)\s*:\s*$/;
-const rScriptJump = /^(?<jump>\s*(?:jump|jumpif\s*\((?<expr>.+)\)))\s+(?<name>[A-Za-z_]\w*)\s*$/;
-const rScriptReturn = /^(?<return>\s*return(?:\s+(?<expr>.+))?)\s*$/;
-const rScriptInclude = /^\s*include\s+(?<delim>')(?<url>(?:\\'|[^'])*)'\s*$/;
-const rScriptIncludeSystem = /^\s*include\s+(?<delim><)(?<url>[^>]*)>\s*$/;
-const rScriptIfBegin = /^\s*if\s+(?<expr>.+)\s*:\s*$/;
-const rScriptIfElseIf = /^\s*elif\s+(?<expr>.+)\s*:\s*$/;
-const rScriptIfElse = /^\s*else\s*:\s*$/;
-const rScriptIfEnd = /^\s*endif\s*$/;
-const rScriptForBegin = /^\s*for\s+(?<value>[A-Za-z_]\w*)(?:\s*,\s*(?<index>[A-Za-z_]\w*))?\s+in\s+(?<values>.+)\s*:\s*$/;
-const rScriptForEnd = /^\s*endfor\s*$/;
-const rScriptWhileBegin = /^\s*while\s+(?<expr>.+)\s*:\s*$/;
-const rScriptWhileEnd = /^\s*endwhile\s*$/;
-const rScriptBreak = /^\s*break\s*$/;
-const rScriptContinue = /^\s*continue\s*$/;
+const rScriptFunctionEnd = new RegExp(`^\\s*endfunction${rPartComment}`);
+const rScriptLabel = new RegExp(`^\\s*(?<name>[A-Za-z_]\\w*)\\s*:${rPartComment}`);
+const rScriptJump = new RegExp(`^(?<jump>\\s*(?:jump|jumpif\\s*\\((?<expr>.+)\\)))\\s+(?<name>[A-Za-z_]\\w*)${rPartComment}`);
+const rScriptReturn = new RegExp(`^(?<return>\\s*return(?:\\s+(?<expr>.+))?)${rPartComment}`);
+const rScriptInclude = new RegExp(`^\\s*include\\s+(?<delim>')(?<url>(?:\\'|[^'])*)'${rPartComment}`);
+const rScriptIncludeSystem = new RegExp(`^\\s*include\\s+(?<delim><)(?<url>[^>]*)>${rPartComment}`);
+const rScriptIfBegin = new RegExp(`^\\s*if\\s+(?<expr>.+)\\s*:${rPartComment}`);
+const rScriptIfElseIf = new RegExp(`^\\s*elif\\s+(?<expr>.+)\\s*:${rPartComment}`);
+const rScriptIfElse = new RegExp(`^\\s*else\\s*:${rPartComment}`);
+const rScriptIfEnd = new RegExp(`^\\s*endif${rPartComment}`);
+const rScriptForBegin = new RegExp(
+    `^\\s*for\\s+(?<value>[A-Za-z_]\\w*)(?:\\s*,\\s*(?<index>[A-Za-z_]\\w*))?\\s+in\\s+(?<values>.+)\\s*:${rPartComment}`
+);
+const rScriptForEnd = new RegExp(`^\\s*endfor${rPartComment}`);
+const rScriptWhileBegin = new RegExp(`^\\s*while\\s+(?<expr>.+)\\s*:${rPartComment}`);
+const rScriptWhileEnd = new RegExp(`^\\s*endwhile${rPartComment}`);
+const rScriptBreak = new RegExp(`^\\s*break${rPartComment}`);
+const rScriptContinue = new RegExp(`^\\s*continue${rPartComment}`);
 
 
 /**
@@ -509,6 +512,11 @@ function parseBinaryExpression(exprText, binLeftExpr = null) {
     // Match a binary operator - if not found, return the left expression
     const matchBinaryOp = binText.match(rExprBinaryOp);
     if (matchBinaryOp === null) {
+        // End-of-line comment?
+        if (binText.match(rExprComment)) {
+            binText = '';
+        }
+
         return [leftExpr, binText];
     }
     const [, binOp] = matchBinaryOp;
@@ -539,18 +547,23 @@ function parseBinaryExpression(exprText, binLeftExpr = null) {
 
 // Binary operator re-order map
 const binaryReorder = {
-    '**': new Set(['*', '/', '%', '+', '-', '<=', '<', '>=', '>', '==', '!=', '&&', '||']),
-    '*': new Set(['+', '-', '<=', '<', '>=', '>', '==', '!=', '&&', '||']),
-    '/': new Set(['+', '-', '<=', '<', '>=', '>', '==', '!=', '&&', '||']),
-    '%': new Set(['+', '-', '<=', '<', '>=', '>', '==', '!=', '&&', '||']),
-    '+': new Set(['<=', '<', '>=', '>', '==', '!=', '&&', '||']),
-    '-': new Set(['<=', '<', '>=', '>', '==', '!=', '&&', '||']),
-    '<=': new Set(['==', '!=', '&&', '||']),
-    '<': new Set(['==', '!=', '&&', '||']),
-    '>=': new Set(['==', '!=', '&&', '||']),
-    '>': new Set(['==', '!=', '&&', '||']),
-    '==': new Set(['&&', '||']),
-    '!=': new Set(['&&', '||']),
+    '**': new Set(['*', '/', '%', '+', '-', '<<', '>>', '<=', '<', '>=', '>', '==', '!=', '&', '^', '|', '&&', '||']),
+    '*': new Set(['+', '-', '<<', '>>', '<=', '<', '>=', '>', '==', '!=', '&', '^', '|', '&&', '||']),
+    '/': new Set(['+', '-', '<<', '>>', '<=', '<', '>=', '>', '==', '!=', '&', '^', '|', '&&', '||']),
+    '%': new Set(['+', '-', '<<', '>>', '<=', '<', '>=', '>', '==', '!=', '&', '^', '|', '&&', '||']),
+    '+': new Set(['<<', '>>', '<=', '<', '>=', '>', '==', '!=', '&', '^', '|', '&&', '||']),
+    '-': new Set(['<<', '>>', '<=', '<', '>=', '>', '==', '!=', '&', '^', '|', '&&', '||']),
+    '<<': new Set(['<=', '<', '>=', '>', '==', '!=', '&', '^', '|', '&&', '||']),
+    '>>': new Set(['<=', '<', '>=', '>', '==', '!=', '&', '^', '|', '&&', '||']),
+    '<=': new Set(['==', '!=', '&', '^', '|', '&&', '||']),
+    '<': new Set(['==', '!=', '&', '^', '|', '&&', '||']),
+    '>=': new Set(['==', '!=', '&', '^', '|', '&&', '||']),
+    '>': new Set(['==', '!=', '&', '^', '|', '&&', '||']),
+    '==': new Set(['&', '^', '|', '&&', '||']),
+    '!=': new Set(['&', '^', '|', '&&', '||']),
+    '&': new Set(['^', '|', '&&', '||']),
+    '^': new Set(['|', '&&', '||']),
+    '|': new Set(['&&', '||']),
     '&&': new Set(['||']),
     '||': new Set([])
 };
@@ -570,6 +583,31 @@ function parseUnaryExpression(exprText) {
         return [{'group': expr}, nextText.slice(matchGroupClose[0].length)];
     }
 
+    // Number?
+    const matchNumber = exprText.match(rExprNumber);
+    if (matchNumber !== null) {
+        const [numberStr] = matchNumber;
+        const number = (numberStr.startsWith('0x') ? parseInt(numberStr, 16) : parseFloat(numberStr));
+        const expr = {'number': number};
+        return [expr, exprText.slice(matchNumber[0].length)];
+    }
+
+    // String?
+    const matchString = exprText.match(rExprString);
+    if (matchString !== null) {
+        const string = matchString[1].replace(rExprStringEscape, '$1');
+        const expr = {'string': string};
+        return [expr, exprText.slice(matchString[0].length)];
+    }
+
+    // String (double quotes)?
+    const matchStringDouble = exprText.match(rExprStringDouble);
+    if (matchStringDouble !== null) {
+        const string = matchStringDouble[1].replace(rExprStringDoubleEscape, '$1');
+        const expr = {'string': string};
+        return [expr, exprText.slice(matchStringDouble[0].length)];
+    }
+
     // Unary operator?
     const matchUnary = exprText.match(rExprUnaryOp);
     if (matchUnary !== null) {
@@ -621,30 +659,6 @@ function parseUnaryExpression(exprText) {
         return [fnExpr, argText];
     }
 
-    // Number?
-    const matchNumber = exprText.match(rExprNumber);
-    if (matchNumber !== null) {
-        const number = parseFloat(matchNumber[1]);
-        const expr = {'number': number};
-        return [expr, exprText.slice(matchNumber[0].length)];
-    }
-
-    // String?
-    const matchString = exprText.match(rExprString);
-    if (matchString !== null) {
-        const string = matchString[1].replace(rExprStringEscape, '$1');
-        const expr = {'string': string};
-        return [expr, exprText.slice(matchString[0].length)];
-    }
-
-    // String (double quotes)?
-    const matchStringDouble = exprText.match(rExprStringDouble);
-    if (matchStringDouble !== null) {
-        const string = matchStringDouble[1].replace(rExprStringDoubleEscape, '$1');
-        const expr = {'string': string};
-        return [expr, exprText.slice(matchStringDouble[0].length)];
-    }
-
     // Variable?
     const matchVariable = exprText.match(rExprVariable);
     if (matchVariable !== null) {
@@ -665,14 +679,15 @@ function parseUnaryExpression(exprText) {
 
 
 // BareScript expression regex
-const rExprBinaryOp = /^\s*(\*\*|\*|\/|%|\+|-|<=|<|>=|>|==|!=|&&|\|\|)/;
-const rExprUnaryOp = /^\s*(!|-)/;
+const rExprComment = /^\s*#.*$/;
+const rExprBinaryOp = /^\s*(\*\*|\*|\/|%|\+|-|<<|>>|<=|<|>=|>|==|!=|&&|\|\||&|\^|\|)/;
+const rExprUnaryOp = /^\s*(!|-|~)/;
 const rExprFunctionOpen = /^\s*([A-Za-z_]\w+)\s*\(/;
 const rExprFunctionSeparator = /^\s*,/;
 const rExprFunctionClose = /^\s*\)/;
 const rExprGroupOpen = /^\s*\(/;
 const rExprGroupClose = /^\s*\)/;
-const rExprNumber = /^\s*([+-]?\d+(?:\.\d*)?(?:e[+-]\d+)?)/;
+const rExprNumber = /^\s*(0x[A-Fa-f0-9]+|[+-]?\d+(?:\.\d*)?(?:e[+-]?\d+)?)/;
 const rExprString = /^\s*'((?:\\\\|\\'|[^'])*)'/;
 const rExprStringEscape = /\\([\\'])/g;
 const rExprStringDouble = /^\s*"((?:\\\\|\\"|[^"])*)"/;
