@@ -5,7 +5,9 @@
 Chisel documentation application
 """
 
-from collections import defaultdict
+import importlib.resources
+from pathlib import PurePosixPath
+import tarfile
 
 from schema_markdown import get_referenced_types
 from schema_markdown.type_model import TYPE_MODEL
@@ -14,34 +16,7 @@ from .action import Action, ActionError
 from .request import RedirectRequest, StaticRequest
 
 
-# The chisel-doc application's HTML stub
-CHISEL_DOC_HTML = b'''\
-<!DOCTYPE html>
-<html lang="en">
-    <head>
-        <title>Chisel</title>
-        <meta charset="UTF-8">
-        <meta name="viewport" content="width=device-width, initial-scale=1">
-        <link rel="stylesheet" href="https://craigahobbs.github.io/markdown-up/app.css">
-    </head>
-    <body>
-    </body>
-    <script type="module">
-        import {MarkdownUp} from 'https://craigahobbs.github.io/markdown-up/lib/app.js';
-        const app = new MarkdownUp(window, {
-            'markdownText': `\
-~~~ markdown-script
-include 'https://craigahobbs.github.io/chisel/doc/app.mds'
-chiselDoc()
-~~~
-`});
-        app.run();
-    </script>
-</html>
-'''
-
-
-def create_doc_requests(requests=None, root_path='/doc', api=True, app=True):
+def create_doc_requests(requests=None, root_path='/doc', api=True, app=True, markdown_up=False):
     """
     Yield a series of requests for use with :meth:`~chisel.Application.add_requests` comprising the Chisel
     documentation application. By default, the documenation application is hosted at "/doc/".
@@ -51,21 +26,49 @@ def create_doc_requests(requests=None, root_path='/doc', api=True, app=True):
     :param str root_path: The documentation application URL root path. The default is "/doc".
     :param bool api: If True, include the documentation APIs. Two documentation APIs are added,
         "/doc/doc_index" and "`/doc/doc_request <doc/#name=chisel_doc_request>`__".
-    :param bool app: If True, include the documentation client application.
+    :param bool app: If True, include the documentation application.
+    :param bool markdown_up: If True, include the MarkdownUp application.
     :returns: Generator of :class:`~chisel.Request`
     """
 
+    root_noslash = root_path.rstrip('/')
+    root_slash = root_noslash + '/'
     if api:
-        yield DocIndex(requests=requests, urls=(('GET', root_path + '/doc_index'),))
-        yield DocRequest(requests=requests, urls=(('GET', root_path + '/doc_request'),))
+        yield DocIndex(requests=requests, urls=(('GET', root_slash + 'doc_index'),))
+        yield DocRequest(requests=requests, urls=(('GET', root_slash + 'doc_request'),))
     if app:
-        yield RedirectRequest((('GET', root_path),), root_path + '/', doc_group='Documentation')
-        yield StaticRequest(
-            'chisel_doc',
-            CHISEL_DOC_HTML,
-            urls=(('GET', root_path + '/'), ('GET', root_path + '/index.html')),
-            doc_group='Documentation'
-        )
+        if root_noslash:
+            yield RedirectRequest((('GET', root_noslash),), root_slash, name='chisel_doc_redirect', doc_group='Documentation')
+        with importlib.resources.files('chisel.static').joinpath('index.html').open('rb') as fh:
+            yield StaticRequest(
+                'chisel_doc',
+                fh.read(),
+                'text/html; charset=utf-8',
+                (('GET', root_slash), ('GET', root_slash + 'index.html')),
+                'The Chisel documentation HTML',
+                'Documentation'
+            )
+        with importlib.resources.files('chisel.static').joinpath('chiselDoc.bare').open('rb') as fh:
+            yield StaticRequest(
+                'chisel_doc_app',
+                fh.read(),
+                'text/plain; charset=utf-8',
+                (('GET', root_slash + 'chiselDoc.bare'),),
+                'The Chisel documentation application',
+                'Documentation'
+            )
+    if markdown_up or app:
+        parent_posix = PurePosixPath(root_path).parent
+        with importlib.resources.files('chisel.static').joinpath('markdown-up.tar.gz').open('rb') as tgz:
+            with tarfile.open(fileobj=tgz, mode='r:gz') as tar:
+                for member in tar.getmembers():
+                    if member.isfile(): # pragma: no branch
+                        yield StaticRequest(
+                            member.name,
+                            tar.extractfile(member).read(),
+                            urls=(('GET', str(parent_posix.joinpath(member.name))),),
+                            doc_group='MarkdownUp Statics'
+                        )
 
 
 class DocIndex(Action):
@@ -106,9 +109,12 @@ action chisel_doc_index
 
     def _doc_index(self, ctx, unused_req):
         requests = self.requests if self.requests is not None else ctx.app.requests
-        groups = defaultdict(list)
+        groups = {}
         for request in requests.values():
-            groups[request.doc_group or 'Uncategorized'].append(request.name)
+            request_group = request.doc_group or 'Uncategorized'
+            if request_group not in groups:
+                groups[request_group] = []
+            groups[request_group].append(request.name)
         return {
             'title': ctx.environ['HTTP_HOST'],
             'groups': {group: sorted(names) for group, names in groups.items()}
@@ -160,6 +166,9 @@ action chisel_doc_request
         # The map of the action's type name's to type models. This member is only present for JSON APIs.
         optional Types types
 
+        # If true, the action has a custom response
+        optional bool custom
+
     errors
         # The request name is unknown
         UnknownName
@@ -186,6 +195,8 @@ action chisel_doc_request
             response['urls'] = [self._url_dict(method, path) for method, path in request.urls]
         if isinstance(request, Action):
             response['types'] = get_referenced_types(request.types, request.name)
+            if request.wsgi_response:
+                response['custom'] = True
         elif request.doc is not None:
             response['doc'] = [request.doc] if isinstance(request.doc, str) else request.doc
 
